/*
 * *##% 
 * Maven helper plugin
 * Copyright (C) 2009 CodeLutin
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Lesser Public License for more details.
 *
 * You should have received a copy of the GNU General Lesser Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/lgpl-3.0.html>.
 * ##%*
 */
package org.nuiton.io.rest;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Arrays;

import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import org.apache.commons.httpclient.Cookie;
import org.apache.commons.httpclient.HostConfiguration;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.HttpConnection;
import org.apache.commons.httpclient.HttpMethod;
import org.apache.commons.httpclient.HttpState;
import org.apache.commons.httpclient.HttpStatus;
import org.apache.commons.httpclient.StatusLine;
import org.apache.commons.httpclient.methods.PostMethod;
import org.apache.commons.httpclient.methods.multipart.FilePart;
import org.apache.commons.httpclient.methods.multipart.MultipartRequestEntity;
import org.apache.commons.httpclient.methods.multipart.Part;
import org.apache.commons.httpclient.methods.multipart.StringPart;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * A REST session.
 * 
 * @author chemit
 * @since 1.0.3
 */
public class RestSession {

    protected HttpClient client;
    protected static final Log log = LogFactory.getLog(RestSession.class);
    protected boolean open;
    protected boolean showRequest;
    protected RestClientConfiguration configuration;

    RestSession(RestClientConfiguration configuration) {
        this.showRequest = configuration.isVerbose();
        this.configuration = configuration;
        this.client = new HttpClient();
        HostConfiguration hostConfiguration = new HostConfiguration();
        hostConfiguration.setHost(configuration.getRestUrl().getHost());
        this.client.setHostConfiguration(hostConfiguration);
        this.client.setState(new HttpState());
    }

    public InputStream askData(RestRequest request) throws RestException, IOException {

        if (!open) {
            throw new IllegalStateException("the session is not opened");
        }
        HttpMethod gm;

        gm = doRequest(request);

        StatusLine sl = gm.getStatusLine();
        int statusCode = sl.getStatusCode();

        if (log.isDebugEnabled()) {
            log.debug("status code " + statusCode + " for " + gm.getPath());
        }

        if (statusCode == HttpStatus.SC_NOT_FOUND) {
            throw new RestDataNotFoundException("could not retreave some datas : " + gm.getResponseBodyAsString());
        }

        if (statusCode != HttpStatus.SC_OK) {
            log.error(gm.getResponseBodyAsString());
            releaseConnection(getConnection());
            throw new RestException("Got error code <" + statusCode + ":" + sl.getReasonPhrase() + "> on " + gm.getPath());
        }
        InputStream stream = gm.getResponseBodyAsStream();
        return stream;
    }

    public InputStream sendData(RestRequest request) throws RestException, IOException {

        if (!open) {
            throw new IllegalStateException("the session is not opened");
        }

        HttpMethod gm = doRequest(request);

        StatusLine sl = gm.getStatusLine();
        int statusCode = sl.getStatusCode();

        if (log.isDebugEnabled()) {
            log.debug("status code " + statusCode + " for " + gm.getPath());
        }

        if (statusCode != HttpStatus.SC_OK) {
            log.error(gm.getResponseBodyAsString());
            releaseConnection(getConnection());
            throw new RestException("Got error code <" + statusCode + ":" + sl.getReasonPhrase() + "> on " + gm.getPath());
        }

        InputStream stream = gm.getResponseBodyAsStream();
        return stream;
    }

    public void setOpen(boolean open) {
        this.open = open;
    }

    public void close() throws IOException {
        client.getState().clearCookies();
        releaseConnection(getConnection());
        this.open = false;
    }

    protected void releaseConnection(HttpConnection connection) {
        if (connection != null && connection.isOpen()) {
            client.getHttpConnectionManager().releaseConnection(connection);
        }
    }

    protected HttpConnection getConnection() {
        return client.getHttpConnectionManager().getConnection(client.getHostConfiguration());
    }

    protected String getUri(String... paths) {
        String uri = client.getHostConfiguration().getHostURL();
        for (String path : paths) {
            uri += '/' + path;
        }
        return uri;
    }

    public HttpMethod doRequest(RestRequest request) throws IOException {

        String uri = getUri(request.getPath());
        String[] parameters = request.getParameters();
        Map<String, File> attachments = request.getAttachments();

        if (showRequest) {
            log.info("doRequest " + uri);
        }
        if (log.isDebugEnabled()) {
            log.debug("doRequest with parameters : " + Arrays.toString(parameters));
        }
        PostMethod gm = new PostMethod(uri);
        if (open) {
            Cookie[] cookies = client.getState().getCookies();
            for (Cookie c : cookies) {
                if (log.isDebugEnabled()) {
                    log.debug("push cookie : " + c.getName() + "=" + c.getValue());
                }
                gm.addRequestHeader("Cookie", c.toExternalForm());
            }
        }

        if (attachments == null) {

            // not a multi-part request

            if (parameters.length > 0) {

                // add parameters

                int nbParams = parameters.length / 2;
                for (int i = 0; i < nbParams; i++) {
                    String key = parameters[2 * i];
                    String value = parameters[2 * i + 1];
                    if (value == null) {
                        if (log.isDebugEnabled()) {
                            log.debug("skip null parameter " + key);
                        }
                        continue;
                    }
                    gm.addParameter(key, value);
                }
            }
        } else {

            // multi-part request

            List<Part> paramParts = new ArrayList<Part>();

            if (parameters.length > 0) {

                // add parameters as part of multi-part

                int nbParams = parameters.length / 2;
                for (int i = 0; i < nbParams; i++) {
                    String key = parameters[2 * i];
                    String value = parameters[2 * i + 1];
                    if (value == null) {
                        if (log.isDebugEnabled()) {
                            log.debug("skip null parameter " + key);
                        }
                        continue;
                    }
                    paramParts.add(new StringPart(key, value));
                }
            }

            // add file parts
            for (Entry<String, File> entry : attachments.entrySet()) {
                String key = entry.getKey();
                File file = entry.getValue();
                paramParts.add(new FilePart(key, file.getName(), file));
            }

            Part[] parts = paramParts.toArray(new Part[paramParts.size()]);

            MultipartRequestEntity entity = new MultipartRequestEntity(parts, gm.getParams());
            gm.setRequestEntity(entity);
//        gm.addRequestHeader("content-type", "multipart/form-data");
            gm.addRequestHeader("content-type", entity.getContentType());
            gm.addRequestHeader("content-length", entity.getContentLength() + "");
            if (showRequest) {
                log.info("content-type : " + entity.getContentType() + ", content-length : " + entity.getContentLength());
            }

        }

        client.executeMethod(gm);

        return gm;
    }
}
